package be.iminds.ilabt.jfed.experiment;

import be.iminds.ilabt.jfed.espec.model.ESpecStep;
import be.iminds.ilabt.jfed.experiment.events.ExperimentEvent;
import be.iminds.ilabt.jfed.experiment.events.ExperimentEventHandler;
import be.iminds.ilabt.jfed.experiment.events.ExperimentEventType;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecLink;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.google.common.util.concurrent.ListenableFuture;
import javafx.stage.Window;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.Instant;
import java.util.Collection;
import java.util.List;

public interface ExperimentController extends ExperimentChangeListener {
    void start();

    ListenableFuture<Void> createSlice();

    ListenableFuture<Void> requestUpdate();

    void shareWithUsers(@Nonnull List<GeniUrn> selectedUsers, boolean registerSshKeys);

    void unshareWithUsers(@Nonnull List<GeniUrn> selectedUsers, boolean unregisterSshKeys);

    void updateSshKeys(@Nonnull List<UserSpec> userspec);

    void reloadOS(Sliver sliver);

    void createDiskImage(FXRspecNode node, String imageName, boolean global, boolean updatePrepare);

    void stop(Collection<ExperimentPart> experimentParts);

    void stop();

    ListenableFuture<Boolean> testLinks();

    void reboot(Sliver sliver);

    void reboot(ExperimentPart experimentPart);

    void openConsole(Sliver sliver);

    void shareLan(FXRspecLink link, String sharedLanName);

    void unshareLan(FXRspecLink link, String sharedLanName);

    ListenableFuture<Collection<UserSpec>> fetchSliceMemberSshKeys();

    ListenableFuture<ExperimentTaskStatus> renew(Instant newExpirationTime);

    void renewParts(Instant newExpirationTime, Collection<ExperimentPart> parts);

    void waitForReady();

    void setupSoftware();

    void rerunESpec(@Nullable ESpecStep startPosition,
                    @Nullable ESpecStep stopPosition);

    @Nonnull
    Experiment getExperiment();

    Window getParentWindow();

    void setParentWindow(Window parentWindow);

    void addListener(ExperimentControllerListener listener);

    void removeListener(ExperimentControllerListener listener);

    ExperimentConnectivityTesterFactory getExperimentConnectivityTesterFactory();

    void setExperimentConnectivityTesterFactory(ExperimentConnectivityTesterFactory experimentConnectivityTesterFactory);
    void setExperimentLinkTesterFactory(ExperimentLinkTesterFactory experimentLinkTesterFactory);

    WaitForReadyTimeoutHandler getWaitForReadyTimeoutHandler();

    void setWaitForReadyTimeoutHandler(WaitForReadyTimeoutHandler waitForReadyTimeoutHandler);

    <T extends ExperimentEvent> void addEventHandler(
            ExperimentEventType<T> type, ExperimentEventHandler<T> eventHandler);
}
